@@ -60,14 +60,14 @@
  */
 
 void construct_handle(const struct ceph_statx *stx, struct Inode *i,
-		      struct export *export, struct handle **obj)
+	struct ceph_export *export, struct ceph_handle **obj)
 {
 	/* Pointer to the handle under construction */
-	struct handle *constructing = NULL;
+	struct ceph_handle *constructing = NULL;
 
 	assert(i);
 
-	constructing = gsh_calloc(1, sizeof(struct handle));
+	constructing = gsh_calloc(1, sizeof(struct ceph_handle));
 
 	constructing->vi.ino.val = stx->stx_ino;
 #ifdef CEPH_NOSNAP
@@ -78,7 +78,7 @@ void construct_handle(const struct ceph_statx *stx, struct Inode *i,
 
 	fsal_obj_handle_init(&constructing->handle, &export->export,
 			     posix2fsal_type(stx->stx_mode));
-	handle_ops_init(&constructing->handle.obj_ops);
+	constructing->handle.obj_ops = &CephFSM.handle_ops;
 	constructing->handle.fsid = posix2fsal_fsid(stx->stx_dev);
 	constructing->handle.fileid = stx->stx_ino;
 
@@ -93,7 +93,7 @@ void construct_handle(const struct ceph_statx *stx, struct Inode *i,
  * @param[in] obj Handle to release
  */
 
-void deconstruct_handle(struct handle *obj)
+void deconstruct_handle(struct ceph_handle *obj)
 {
 	ceph_ll_put(obj->export->cmount, obj->i);
 	fsal_obj_handle_fini(&obj->handle);
