@@ -946,6 +946,7 @@ static int pxy_setsessionid(sessionid4 new_sessionid, uint32_t *lease_time,
 	sequenceid4 seqid;
 	CREATE_SESSION4res *s_res;
 	CREATE_SESSION4resok *res_ok;
+	callback_sec_parms4 sec_parms4;
 	uint32_t fore_ca_rdma_ird_val_sink;
 	uint32_t back_ca_rdma_ird_val_sink;
 
@@ -963,7 +964,7 @@ static int pxy_setsessionid(sessionid4 new_sessionid, uint32_t *lease_time,
 						&back_ca_rdma_ird_val_sink;
 
 	COMPOUNDV4_ARG_ADD_OP_CREATE_SESSION(opcnt, arg, cid, seqid,
-					     (&(pxy_exp->info)));
+					     (&(pxy_exp->info)), &sec_parms4);
 	rc = pxy_compoundv4_execute(__func__, NULL, opcnt, arg, res, pxy_exp);
 	if (rc != NFS4_OK)
 		return -1;
@@ -1033,7 +1034,7 @@ static int pxy_setclientid(clientid4 *new_clientid, sequenceid4 *new_seqid,
 	clientid.co_ownerid.co_ownerid_val = clientid_name;
 
 	/* copy to intermediate uint64_t to 0-fill or truncate as needed */
-	temp_verifier = (uint64_t)ServerBootTime.tv_sec;
+	temp_verifier = (uint64_t)nfs_ServerBootTime.tv_sec;
 	BUILD_BUG_ON(sizeof(clientid.co_verifier) != sizeof(uint64_t));
 	memcpy(&clientid.co_verifier, &temp_verifier, sizeof(uint64_t));
 
@@ -1363,7 +1364,7 @@ static fsal_status_t pxy_lookup_impl(struct fsal_obj_handle *parent,
 	int rc;
 	uint32_t opcnt = 0;
 	GETATTR4resok *atok;
-	GETATTR4resok *atok_per_file_system_attr;
+	GETATTR4resok *atok_per_file_system_attr = NULL;
 	GETFH4resok *fhok;
 	sessionid4 sid;
 	/* SEQUENCE PUTROOTFH/PUTFH LOOKUP GETFH GETATTR (GETATTR) */
@@ -2890,6 +2891,14 @@ fsal_status_t pxy_lookup_path(struct fsal_export *exp_hdl,
 	pcopy = gsh_strdup(path);
 
 	p = strtok_r(pcopy, "/", &saved);
+	if (!p) {
+		fsal_status_t st = pxy_lookup_impl(parent, exp_hdl, creds,
+						   NULL, &next, attrs_out);
+		if (FSAL_IS_ERROR(st)) {
+			gsh_free(pcopy);
+			return st;
+		}
+	}
 	while (p) {
 		if (strcmp(p, "..") == 0) {
 			/* Don't allow lookup of ".." */
