@@ -39,78 +39,56 @@
 #include "pseudofs_methods.h"
 #include "../fsal_private.h"
 
-/* PSEUDOFS FSAL module private storage
- */
-
-struct pseudo_fsal_module {
-	struct fsal_module fsal;
-	struct fsal_staticfsinfo_t fs_info;
-	/* pseudofsfs_specific_initinfo_t specific_info;  placeholder */
-};
-
 const char pseudoname[] = "PSEUDO";
 
-/* filesystem info for PSEUDOFS */
-static struct fsal_staticfsinfo_t default_posix_info = {
-	.maxfilesize = UINT64_MAX,
-	.maxlink = 0,
-	.maxnamelen = MAXNAMLEN,
-	.maxpathlen = MAXPATHLEN,
-	.no_trunc = true,
-	.chown_restricted = true,
-	.case_insensitive = false,
-	.case_preserving = true,
-	.link_support = false,
-	.symlink_support = false,
-	.lock_support = false,
-	.lock_support_async_block = false,
-	.named_attr = false,
-	.unique_handles = true,
-	.lease_time = {10, 0},
-	.acl_support = 0,
-	.cansettime = true,
-	.homogenous = true,
-	.supported_attrs = PSEUDO_SUPPORTED_ATTRS,
-	.maxread = FSAL_MAXIOSIZE,
-	.maxwrite = FSAL_MAXIOSIZE,
-	.umask = 0,
-	.auth_exportpath_xdev = false,
-	.xattr_access_rights = 0400,	/* root=RW, owner=R */
-	.link_supports_permission_checks = false,
+/* my module private storage
+ */
+
+struct pseudo_fsal_module PSEUDOFS = {
+	.module = {
+		.fs_info = {
+			.maxfilesize = 512,
+			.maxlink = 0,
+			.maxnamelen = MAXNAMLEN,
+			.maxpathlen = MAXPATHLEN,
+			.no_trunc = true,
+			.chown_restricted = true,
+			.case_insensitive = false,
+			.case_preserving = true,
+			.link_support = false,
+			.symlink_support = false,
+			.lock_support = false,
+			.lock_support_async_block = false,
+			.named_attr = false,
+			.unique_handles = true,
+			.acl_support = 0,
+			.cansettime = true,
+			.homogenous = true,
+			.supported_attrs = PSEUDO_SUPPORTED_ATTRS,
+			.maxread = FSAL_MAXIOSIZE,
+			.maxwrite = FSAL_MAXIOSIZE,
+			.umask = 0,
+			.auth_exportpath_xdev = false,
+			.link_supports_permission_checks = false,
+		}
+	}
 };
 
 /* private helper for export object
  */
 
-struct fsal_staticfsinfo_t *pseudofs_staticinfo(struct fsal_module *hdl)
-{
-	struct pseudo_fsal_module *myself;
-
-	myself = container_of(hdl, struct pseudo_fsal_module, fsal);
-	return &myself->fs_info;
-}
-
 /* Initialize pseudo fs info */
-static void init_config(struct fsal_module *fsal_hdl)
+static void init_config(struct fsal_module *pseudo_fsal_module)
 {
-	struct pseudo_fsal_module *pseudofs_me =
-	    container_of(fsal_hdl, struct pseudo_fsal_module, fsal);
-
-	/* get a copy of the defaults */
-	pseudofs_me->fs_info = default_posix_info;
-
 	/* if we have fsal specific params, do them here
 	 * fsal_hdl->name is used to find the block containing the
 	 * params.
 	 */
 
-	display_fsinfo(&pseudofs_me->fs_info);
-	LogFullDebug(COMPONENT_FSAL,
-		     "Supported attributes default = 0x%" PRIx64,
-		     default_posix_info.supported_attrs);
+	display_fsinfo(pseudo_fsal_module);
 	LogDebug(COMPONENT_FSAL,
 		 "FSAL INIT: Supported attributes mask = 0x%" PRIx64,
-		 pseudofs_me->fs_info.supported_attrs);
+		 pseudo_fsal_module->fs_info.supported_attrs);
 }
 
 /* Module initialization.
@@ -118,11 +96,6 @@ static void init_config(struct fsal_module *fsal_hdl)
  * keep a private pointer to me in myself
  */
 
-/* my module private storage
- */
-
-static struct pseudo_fsal_module PSEUDOFS;
-
 /* linkage to the exports and handle ops initializers
  */
 
@@ -130,7 +103,7 @@ int unload_pseudo_fsal(struct fsal_module *fsal_hdl)
 {
 	int retval;
 
-	retval = unregister_fsal(&PSEUDOFS.fsal);
+	retval = unregister_fsal(&PSEUDOFS.module);
 	if (retval != 0)
 		fprintf(stderr, "PSEUDO module failed to unregister");
 
@@ -140,7 +113,7 @@ int unload_pseudo_fsal(struct fsal_module *fsal_hdl)
 void pseudo_fsal_init(void)
 {
 	int retval;
-	struct fsal_module *myself = &PSEUDOFS.fsal;
+	struct fsal_module *myself = &PSEUDOFS.module;
 
 	retval = register_fsal(myself, pseudoname, FSAL_MAJOR_VERSION,
 			       FSAL_MINOR_VERSION, FSAL_ID_NO_PNFS);
@@ -151,6 +124,9 @@ void pseudo_fsal_init(void)
 	myself->m_ops.create_export = pseudofs_create_export;
 	myself->m_ops.unload = unload_pseudo_fsal;
 
+	/* Initialize the fsal_obj_handle ops for FSAL PSEUDO */
+	pseudofs_handle_ops_init(&PSEUDOFS.handle_ops);
+
 	/* initialize our config */
 	init_config(myself);
 }
