@@ -6,6 +6,7 @@
  *
  * contributeur : Philippe DENIEL   philippe.deniel@cea.fr
  *                Thomas LEIBOVICI  thomas.leibovici@cea.fr
+ *                Patrice LUCAS     patrice.lucas@cea.fr
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -39,6 +40,7 @@
 #include "gsh_list.h"
 #include "fsal.h"
 #include "FSAL/fsal_init.h"
+#include "vfs_methods.h"
 
 /* PANFS FSAL module private storage
  */
@@ -46,54 +48,55 @@
 /* defined the set of attributes supported with POSIX */
 #define PANFS_SUPPORTED_ATTRIBUTES ((const attrmask_t) (ATTRS_POSIX | ATTR_ACL))
 
-struct panfs_fsal_module {
-	struct fsal_module fsal;
-	struct fsal_staticfsinfo_t fs_info;
-	/* panfs_specific_initinfo_t specific_info;  placeholder */
-};
-
 const char myname[] = "PANFS";
 
-/* filesystem info for PANFS */
-static struct fsal_staticfsinfo_t default_posix_info = {
-	.maxfilesize = UINT64_MAX,
-	.maxlink = _POSIX_LINK_MAX,
-	.maxnamelen = 1024,
-	.maxpathlen = 1024,
-	.no_trunc = true,
-	.chown_restricted = true,
-	.case_insensitive = false,
-	.case_preserving = true,
-	.lock_support = false,
-	.lock_support_async_block = false,
-	.named_attr = true,
-	.unique_handles = true,
-	.lease_time = {10, 0},
-	.acl_support = FSAL_ACLSUPPORT_ALLOW | FSAL_ACLSUPPORT_DENY,
-	.homogenous = true,
-	.supported_attrs = PANFS_SUPPORTED_ATTRIBUTES,
-	.maxread = FSAL_MAXIOSIZE,
-	.maxwrite = FSAL_MAXIOSIZE,
-	.link_supports_permission_checks = false,
+/* my module private storage
+ */
+
+static struct vfs_fsal_module PANFS = {
+	.module = {
+		.fs_info = {
+			.maxfilesize = INT64_MAX,
+			.maxlink = _POSIX_LINK_MAX,
+			.maxnamelen = 1024,
+			.maxpathlen = 1024,
+			.no_trunc = true,
+			.chown_restricted = true,
+			.case_insensitive = false,
+			.case_preserving = true,
+			.lock_support = false,
+			.lock_support_async_block = false,
+			.named_attr = true,
+			.unique_handles = true,
+			.acl_support = FSAL_ACLSUPPORT_ALLOW |
+						FSAL_ACLSUPPORT_DENY,
+			.homogenous = true,
+			.supported_attrs = PANFS_SUPPORTED_ATTRIBUTES,
+			.maxread = FSAL_MAXIOSIZE,
+			.maxwrite = FSAL_MAXIOSIZE,
+			.link_supports_permission_checks = false,
+		}
+	},
+	.only_one_user = false
 };
 
 static struct config_item panfs_params[] = {
-	CONF_ITEM_BOOL("link_support", true,
-		       fsal_staticfsinfo_t, link_support),
-	CONF_ITEM_BOOL("symlink_support", true,
-		       fsal_staticfsinfo_t, symlink_support),
-	CONF_ITEM_BOOL("cansettime", true,
-		       fsal_staticfsinfo_t, cansettime),
+	CONF_ITEM_BOOL("link_support", true, vfs_fsal_module,
+		       module.fs_info.link_support),
+	CONF_ITEM_BOOL("symlink_support", true, vfs_fsal_module,
+		       module.fs_info.symlink_support),
+	CONF_ITEM_BOOL("cansettime", true, vfs_fsal_module,
+		       module.fs_info.cansettime),
 	CONF_ITEM_UI64("maxread", 512, FSAL_MAXIOSIZE, FSAL_MAXIOSIZE,
-		       fsal_staticfsinfo_t, maxread),
+		       vfs_fsal_module, module.fs_info.maxread),
 	CONF_ITEM_UI64("maxwrite", 512, FSAL_MAXIOSIZE, FSAL_MAXIOSIZE,
-		       fsal_staticfsinfo_t, maxwrite),
-	CONF_ITEM_MODE("umask", 0,
-		       fsal_staticfsinfo_t, umask),
-	CONF_ITEM_BOOL("auth_xdev_export", false,
-		       fsal_staticfsinfo_t, auth_exportpath_xdev),
-	CONF_ITEM_MODE("xattr_access_rights", 0400,
-		       fsal_staticfsinfo_t, xattr_access_rights),
+		       vfs_fsal_module, module.fs_info.maxwrite),
+	CONF_ITEM_MODE("umask", 0, vfs_fsal_module,
+		       module.fs_info.umask),
+	CONF_ITEM_BOOL("auth_xdev_export", false, vfs_fsal_module,
+		       module.fs_info.auth_exportpath_xdev),
+	CONF_ITEM_BOOL("only_one_user", false, vfs_fsal_module,
+		       only_one_user),
 	CONFIG_EOL
 };
 
@@ -106,17 +109,6 @@ struct config_block panfs_param = {
 	.blk_desc.u.blk.commit = noop_conf_commit
 };
 
-/* private helper for export object
- */
-
-struct fsal_staticfsinfo_t *vfs_staticinfo(struct fsal_module *hdl)
-{
-	struct panfs_fsal_module *myself;
-
-	myself = container_of(hdl, struct panfs_fsal_module, fsal);
-	return &myself->fs_info;
-}
-
 /* Module methods
  */
 
@@ -124,31 +116,32 @@ struct fsal_staticfsinfo_t *vfs_staticinfo(struct fsal_module *hdl)
  * must be called with a reference taken (via lookup_fsal)
  */
 
-static fsal_status_t init_config(struct fsal_module *fsal_hdl,
+static fsal_status_t init_config(struct fsal_module *panfs_fsal_module,
 				 config_file_t config_struct,
 				 struct config_error_type *err_type)
 {
-	struct panfs_fsal_module *panfs_me =
-	    container_of(fsal_hdl, struct panfs_fsal_module, fsal);
+	struct vfs_fsal_module *panfs_module =
+	    container_of(panfs_fsal_module, struct vfs_fsal_module, module);
+
+	LogFullDebug(COMPONENT_FSAL,
+		"Supported attributes default = 0x%" PRIx64,
+		panfs_module->module.fs_info.supported_attrs);
 
-	panfs_me->fs_info = default_posix_info;	/* copy the consts */
 	(void) load_config_from_parse(config_struct,
 				      &panfs_param,
-				      &panfs_me->fs_info,
+				      panfs_module,
 				      true,
 				      err_type);
 	if (!config_error_is_harmless(err_type))
 		return fsalstat(ERR_FSAL_INVAL, 0);
-	display_fsinfo(&panfs_me->fs_info);
+	display_fsinfo(&panfs_module->module);
 	LogFullDebug(COMPONENT_FSAL,
 		     "Supported attributes constant = 0x%" PRIx64,
 		     PANFS_SUPPORTED_ATTRIBUTES);
-	LogFullDebug(COMPONENT_FSAL,
-		     "Supported attributes default = 0x%" PRIx64,
-		     default_posix_info.supported_attrs);
+
 	LogDebug(COMPONENT_FSAL,
 		 "FSAL INIT: Supported attributes mask = 0x%" PRIx64,
-		 panfs_me->fs_info.supported_attrs);
+		 panfs_module->module.fs_info.supported_attrs);
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
@@ -165,18 +158,13 @@ fsal_status_t vfs_create_export(struct fsal_module *fsal_hdl,
  * keep a private pointer to me in myself
  */
 
-/* my module private storage
- */
-
-static struct panfs_fsal_module PANFS;
-
 /* linkage to the exports and handle ops initializers
  */
 
 MODULE_INIT void panfs_init(void)
 {
 	int retval;
-	struct fsal_module *myself = &PANFS.fsal;
+	struct fsal_module *myself = &PANFS.module;
 
 	retval = register_fsal(myself, myname, FSAL_MAJOR_VERSION,
 			       FSAL_MINOR_VERSION, FSAL_ID_PANFS);
@@ -192,7 +180,7 @@ MODULE_FINI void panfs_unload(void)
 {
 	int retval;
 
-	retval = unregister_fsal(&PANFS.fsal);
+	retval = unregister_fsal(&PANFS.module);
 	if (retval != 0) {
 		fprintf(stderr, "PANFS module failed to unregister");
 		return;
