@@ -234,15 +234,10 @@ int load_fsal(const char *name,
 
 	PTHREAD_MUTEX_lock(&fsal_lock);
 	if (dl == NULL) {
-#ifdef ELIBACC
-		retval = ELIBACC;	/* hand craft a meaningful error */
-#else
-		retval = EPERM;	/* ELIBACC does not exist on MacOS */
-#endif
-		dl_error = gsh_strdup(dlerror());
-		LogCrit(COMPONENT_INIT, "Could not dlopen module:%s Error:%s",
-			path, dl_error);
-		goto errout;
+		dl_error = dlerror();
+		LogFatal(COMPONENT_INIT,
+			 "Could not dlopen module: %s Error: %s. You might want to install the nfs-ganesha-%s package",
+			 path, dl_error, name);
 	}
 	dlerror();	/* clear it */
 
@@ -425,6 +420,7 @@ int register_fsal(struct fsal_module *fsal_hdl, const char *name,
 		PTHREAD_RWLOCK_PREFER_WRITER_NONRECURSIVE_NP);
 #endif
 	PTHREAD_RWLOCK_init(&fsal_hdl->lock, &attrs);
+	pthread_rwlockattr_destroy(&attrs);
 	glist_init(&fsal_hdl->servers);
 	glist_init(&fsal_hdl->handles);
 	glist_init(&fsal_hdl->exports);
