@@ -95,7 +95,7 @@ int _9p_walk(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 		pnewfid->fid = *newfid;
 
 		/* Increments refcount */
-		pnewfid->pentry->obj_ops.get_ref(pnewfid->pentry);
+		pnewfid->pentry->obj_ops->get_ref(pnewfid->pentry);
 	} else {
 		/* the walk is in fact a lookup */
 		pentry = pfid->pentry;
@@ -129,7 +129,7 @@ int _9p_walk(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 			}
 
 			if (pentry != pfid->pentry)
-				pentry->obj_ops.put_ref(pentry);
+				pentry->obj_ops->put_ref(pentry);
 
 			pentry = pnewfid->pentry;
 		}
@@ -147,7 +147,7 @@ int _9p_walk(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 
 		/* Refcounted object (incremented at the end of the function,
 		 * if there was no errors). */
-		pnewfid->export = pfid->export;
+		pnewfid->fid_export = pfid->fid_export;
 		pnewfid->ucred = pfid->ucred;
 
 		/* Build the qid */
@@ -156,8 +156,7 @@ int _9p_walk(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 		pnewfid->qid.version = 0;
 		pnewfid->qid.path = pnewfid->pentry->fileid;
 
-		pnewfid->specdata.xattr.xattr_id = 0;
-		pnewfid->specdata.xattr.xattr_content = NULL;
+		pnewfid->xattr = NULL;
 
 		switch (pnewfid->pentry->type) {
 		case REGULAR_FILE:
@@ -179,7 +178,7 @@ int _9p_walk(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 		default:
 			LogMajor(COMPONENT_9P,
 				 "implementation error, you should not see this message !!!!!!");
-			pentry->obj_ops.put_ref(pentry);
+			pentry->obj_ops->put_ref(pentry);
 			gsh_free(pnewfid);
 			return _9p_rerror(req9p, msgtag, EINVAL,
 					  plenout, preply);
@@ -192,10 +191,10 @@ int _9p_walk(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 	 * to one to represent the state_t being embeded in the fid. This
 	 * prevents it from ever being reduced to zero by dec_state_t_ref.
 	 */
-	pnewfid->state = pnewfid->export->fsal_export->exp_ops.alloc_state(
-						pnewfid->export->fsal_export,
-						STATE_TYPE_9P_FID,
-						NULL);
+	pnewfid->state = pnewfid->fid_export->fsal_export->exp_ops.alloc_state(
+					       pnewfid->fid_export->fsal_export,
+					       STATE_TYPE_9P_FID,
+					       NULL);
 
 	glist_init(&pnewfid->state->state_data.fid.state_locklist);
 	pnewfid->state->state_refcount = 1;
@@ -209,11 +208,11 @@ int _9p_walk(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 	/* Increment refcounters. */
 	uid2grp_hold_group_data(pnewfid->gdata);
 	get_9p_user_cred_ref(pnewfid->ucred);
-	get_gsh_export_ref(pnewfid->export);
+	get_gsh_export_ref(pnewfid->fid_export);
 
 	if (pnewfid->ppentry != NULL) {
 		/* Increments refcount for ppentry */
-		pnewfid->ppentry->obj_ops.get_ref(pnewfid->ppentry);
+		pnewfid->ppentry->obj_ops->get_ref(pnewfid->ppentry);
 	}
 
 	/* Build the reply */
