@@ -55,6 +55,8 @@ static const char *lock_tag = "LOCK";
  *
  */
 
+#define SUCCESS_RESP_SIZE (sizeof(nfsstat4) + sizeof(stateid4))
+
 int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		 struct nfs_resop4 *resp)
 {
@@ -101,13 +103,23 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	int rc;
 	struct fsal_obj_handle *obj = data->current_obj;
 	bool state_lock_held = false;
+	uint64_t maxfilesize =
+	    op_ctx->fsal_export->exp_ops.fs_maxfilesize(op_ctx->fsal_export);
 
 	LogDebug(COMPONENT_NFS_V4_LOCK,
 		 "Entering NFS v4 LOCK handler ----------------------");
 
 	/* Initialize to sane starting values */
 	resp->resop = NFS4_OP_LOCK;
-	res_LOCK4->status = NFS4_OK;
+
+	/* Before starting, make sure we have room for succeseful response so
+	 * we don't have to undo a successful lock operation (that may not be
+	 * reversible if it overlaps an existing lock).
+	 */
+	res_LOCK4->status = check_resp_room(data, SUCCESS_RESP_SIZE);
+
+	if (res_LOCK4->status != NFS4_OK)
+		return res_LOCK4->status;
 
 	/* Record the sequence info */
 	if (data->minorversion > 0) {
@@ -387,7 +399,24 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	if (lock_desc.lock_length >
 	    (STATE_LOCK_OFFSET_EOF - lock_desc.lock_start)) {
 		res_LOCK4->status = NFS4ERR_INVAL;
-		LogDebug(COMPONENT_NFS_V4_LOCK, "LOCK failed length overflow");
+		LogDebug(COMPONENT_NFS_V4_LOCK,
+			 "LOCK failed length overflow start %"PRIx64
+			 " length %"PRIx64,
+			 lock_desc.lock_start, lock_desc.lock_length);
+		goto out;
+	}
+
+	/* Check for range overflow past maxfilesize.  Comparing beyond 2^64 is
+	 * not possible in 64 bits precision, but off+len > maxfilesize is
+	 * equivalent to len > maxfilesize - off
+	 */
+	if (lock_desc.lock_length > (maxfilesize - lock_desc.lock_start)) {
+		res_LOCK4->status = NFS4ERR_BAD_RANGE;
+		LogDebug(COMPONENT_NFS_V4_LOCK,
+			 "LOCK failed past maxfilesize %"PRIx64" start %"PRIx64
+			 " length %"PRIx64,
+			 maxfilesize,
+			 lock_desc.lock_start, lock_desc.lock_length);
 		goto out;
 	}
 
@@ -585,18 +614,22 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 				  &conflict_desc);
 
 	if (state_status != STATE_SUCCESS) {
-		if (state_status == STATE_LOCK_CONFLICT) {
-			/* A conflicting lock from a different
-			   lock_owner, returns NFS4ERR_DENIED */
-			Process_nfs4_conflict(&res_LOCK4->LOCK4res_u.denied,
-					      conflict_owner,
-					      &conflict_desc);
-		}
-
 		LogDebug(COMPONENT_NFS_V4_LOCK, "LOCK failed with status %s",
 			 state_err_str(state_status));
 
-		res_LOCK4->status = nfs4_Errno_state(state_status);
+		if (state_status == STATE_LOCK_CONFLICT) {
+			/* A conflicting lock from a different lock_owner,
+			 * returns NFS4ERR_DENIED, but check that the
+			 * response will fit, if not, return response error.
+			 */
+			res_LOCK4->status = Process_nfs4_conflict(
+						&res_LOCK4->LOCK4res_u.denied,
+						conflict_owner,
+						&conflict_desc,
+						data);
+		} else {
+			res_LOCK4->status = nfs4_Errno_state(state_status);
+		}
 
 		/* Save the response in the lock or open owner */
 		if (res_LOCK4->status != NFS4ERR_RESOURCE
@@ -622,6 +655,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		op_ctx->clientid = NULL;
 
 	res_LOCK4->status = NFS4_OK;
+	data->op_resp_size = SUCCESS_RESP_SIZE;
 
 	/* Handle stateid/seqid for success */
 	update_stateid(lock_state,
