@@ -83,6 +83,8 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 	int rc;
 	/* stateid if available matching owner and entry */
 	state_t *state;
+	uint64_t maxfilesize =
+	    op_ctx->fsal_export->exp_ops.fs_maxfilesize(op_ctx->fsal_export);
 
 	LogDebug(COMPONENT_NFS_V4_LOCK,
 		 "Entering NFS v4 LOCKT handler ----------------------------");
@@ -140,6 +142,24 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 	if (lock_desc.lock_length >
 	    (STATE_LOCK_OFFSET_EOF - lock_desc.lock_start)) {
 		res_LOCKT4->status = NFS4ERR_INVAL;
+		LogDebug(COMPONENT_NFS_V4_LOCK,
+			 "LOCK failed length overflow start %"PRIx64
+			 " length %"PRIx64,
+			 lock_desc.lock_start, lock_desc.lock_length);
+		return res_LOCKT4->status;
+	}
+
+	/* Check for range overflow past maxfilesize.  Comparing beyond 2^64 is
+	 * not possible in 64 bits precision, but off+len > maxfilesize is
+	 * equivalent to len > maxfilesize - off
+	 */
+	if (lock_desc.lock_length > (maxfilesize - lock_desc.lock_start)) {
+		res_LOCKT4->status = NFS4ERR_BAD_RANGE;
+		LogDebug(COMPONENT_NFS_V4_LOCK,
+			 "LOCK failed past maxfilesize %"PRIx64" start %"PRIx64
+			 " length %"PRIx64,
+			 maxfilesize,
+			 lock_desc.lock_start, lock_desc.lock_length);
 		return res_LOCKT4->status;
 	}
 
@@ -214,9 +234,17 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 		 */
 		LogStateOwner("Conflict: ", conflict_owner);
 
-		Process_nfs4_conflict(&res_LOCKT4->LOCKT4res_u.denied,
-				      conflict_owner,
-				      &conflict_desc);
+		res_LOCKT4->status = Process_nfs4_conflict(
+						&res_LOCKT4->LOCKT4res_u.denied,
+						conflict_owner,
+						&conflict_desc,
+						data);
+	} else {
+		/* Return result */
+		res_LOCKT4->status = nfs4_Errno_state(state_status);
+
+		/* response is just nfsstat4 */
+		data->op_resp_size = sizeof(nfsstat4);
 	}
 
 	if (data->minorversion == 0)
@@ -229,9 +257,6 @@ int nfs4_op_lockt(struct nfs_argop4 *op, compound_data_t *data,
 	if (state != NULL)
 		dec_state_t_ref(state);
 
-	/* Return result */
-	res_LOCKT4->status = nfs4_Errno_state(state_status);
-
  out:
 
 	/* Update the lease before exit */
