@@ -202,7 +202,7 @@ int nfs4_op_lookup(struct nfs_argop4 *op, compound_data_t *data,
 				op_ctx->ctx_export->pseudopath,
 				op_ctx->ctx_export->export_id, name);
 
-			file_obj->obj_ops.put_ref(file_obj);
+			file_obj->obj_ops->put_ref(file_obj);
 			file_obj = obj;
 		} else {
 			PTHREAD_RWLOCK_unlock(&file_obj->state_hdl->state_lock);
@@ -220,7 +220,7 @@ int nfs4_op_lookup(struct nfs_argop4 *op, compound_data_t *data,
 	set_current_entry(data, file_obj);
 
 	/* Put our ref */
-	file_obj->obj_ops.put_ref(file_obj);
+	file_obj->obj_ops->put_ref(file_obj);
 	file_obj = NULL;
 
 	/* Return successfully */
@@ -229,7 +229,7 @@ int nfs4_op_lookup(struct nfs_argop4 *op, compound_data_t *data,
  out:
 	/* Release reference on file_obj if we didn't utilze it. */
 	if (file_obj)
-		file_obj->obj_ops.put_ref(file_obj);
+		file_obj->obj_ops->put_ref(file_obj);
 
 	gsh_free(name);
 
