@@ -85,7 +85,7 @@ static nfsstat4 open4_create_fh(compound_data_t *data,
 	/* Building a new fh */
 	if (!nfs4_FSALToFhandle(false, &data->currentFH, obj,
 					op_ctx->ctx_export)) {
-		obj->obj_ops.put_ref(obj);
+		obj->obj_ops->put_ref(obj);
 		return NFS4ERR_SERVERFAULT;
 	}
 
@@ -102,7 +102,7 @@ static nfsstat4 open4_create_fh(compound_data_t *data,
 	}
 
 	/* Put our ref */
-	obj->obj_ops.put_ref(obj);
+	obj->obj_ops->put_ref(obj);
 
 	if (set_no_cleanup) {
 		/* And clear the no_cleanup we set above. */
@@ -470,18 +470,17 @@ static void get_delegation(compound_data_t *data, OPEN4args *args,
 			 state_err_str(state_status));
 		whynone->ond_why = WND4_RESOURCE;
 		return;
-	} else {
-		new_state->state_seqid++;
+	}
+	new_state->state_seqid++;
 
-		LogFullDebugOpaque(COMPONENT_STATE,
-				   "delegation state added, stateid: %s",
-				   100, new_state->stateid_other, OTHERSIZE);
+	LogFullDebugOpaque(COMPONENT_STATE,
+			   "delegation state added, stateid: %s",
+			   100, new_state->stateid_other, OTHERSIZE);
 
-		/* acquire_lease_lock() gets the delegation from FSAL */
-		state_status = acquire_lease_lock(ostate,
-						  clientowner,
-						  new_state);
-		if (state_status != STATE_SUCCESS) {
+	/* acquire_lease_lock() gets the delegation from FSAL */
+	state_status = acquire_lease_lock(ostate, clientowner, new_state);
+	if (state_status != STATE_SUCCESS) {
+		if (args->claim.claim != CLAIM_PREVIOUS) {
 			LogDebug(COMPONENT_NFS_V4_LOCK,
 				 "get delegation call added state but failed to lock with status %s",
 				 state_err_str(state_status));
@@ -492,27 +491,26 @@ static void get_delegation(compound_data_t *data, OPEN4args *args,
 			else
 				whynone->ond_why = WND4_RESOURCE;
 			return;
-		} else {
-			nfs_space_limit4 *space_limit = &writeres->space_limit;
-
-			resok->delegation.delegation_type = deleg_type;
-			ostate->file.fdeleg_stats.fds_deleg_type = deleg_type;
-			if (deleg_type == OPEN_DELEGATE_WRITE) {
-				space_limit->limitby = NFS_LIMIT_SIZE;
-				space_limit->nfs_space_limit4_u.filesize =
-						DELEG_SPACE_LIMIT_FILESZ;
-				COPY_STATEID(&writeres->stateid, new_state);
-				writeres->recall = prerecall;
-				get_deleg_perm(&writeres->permissions,
-					       deleg_type);
-			} else {
-				assert(deleg_type == OPEN_DELEGATE_READ);
-				COPY_STATEID(&readres->stateid, new_state);
-				readres->recall = prerecall;
-				get_deleg_perm(&readres->permissions,
-					       deleg_type);
-			}
 		}
+		prerecall = true;
+	}
+
+	resok->delegation.delegation_type = deleg_type;
+	ostate->file.fdeleg_stats.fds_deleg_type = deleg_type;
+	if (deleg_type == OPEN_DELEGATE_WRITE) {
+		nfs_space_limit4 *space_limit = &writeres->space_limit;
+
+		space_limit->limitby = NFS_LIMIT_SIZE;
+		space_limit->nfs_space_limit4_u.filesize =
+				DELEG_SPACE_LIMIT_FILESZ;
+		COPY_STATEID(&writeres->stateid, new_state);
+		writeres->recall = prerecall;
+		get_deleg_perm(&writeres->permissions, deleg_type);
+	} else {
+		assert(deleg_type == OPEN_DELEGATE_READ);
+		COPY_STATEID(&readres->stateid, new_state);
+		readres->recall = prerecall;
+		get_deleg_perm(&readres->permissions, deleg_type);
 	}
 
 	if (isDebug(COMPONENT_NFS_V4_LOCK)) {
@@ -725,6 +723,7 @@ static void open4_ex(OPEN4args *arg,
 
 	/* Make sure... */
 	*file_state = NULL;
+	*new_state = false;
 
 	/* Pre-process the claim type */
 	switch (claim) {
@@ -977,10 +976,10 @@ static void open4_ex(OPEN4args *arg,
 		}
 
 		/* We need an extra reference below. */
-		file_obj->obj_ops.get_ref(file_obj);
+		file_obj->obj_ops->get_ref(file_obj);
 	} else {
 		old_openflags =
-			file_obj->obj_ops.status2(file_obj, *file_state);
+			file_obj->obj_ops->status2(file_obj, *file_state);
 
 		/* Open upgrade */
 		LogFullDebug(COMPONENT_STATE, "Calling reopen2");
@@ -995,7 +994,7 @@ static void open4_ex(OPEN4args *arg,
 		}
 
 		/* We need an extra reference below. */
-		file_obj->obj_ops.get_ref(file_obj);
+		file_obj->obj_ops->get_ref(file_obj);
 	}
 
 	if (file_obj == NULL) {
@@ -1055,7 +1054,7 @@ static void open4_ex(OPEN4args *arg,
 			 * release the ref on file_obj, since the state add
 			 * failed.
 			 */
-			file_obj->obj_ops.put_ref(file_obj);
+			file_obj->obj_ops->put_ref(file_obj);
 			res_OPEN4->status = nfs4_Errno_state(state_status);
 			*new_state = false;
 			goto out;
@@ -1074,7 +1073,7 @@ static void open4_ex(OPEN4args *arg,
 			*new_state = false;
 		} else {
 			/*Do an open downgrade to the old open flags */
-			status = file_obj->obj_ops.reopen2(file_obj,
+			status = file_obj->obj_ops->reopen2(file_obj,
 							   *file_state,
 							   old_openflags);
 			if (FSAL_IS_ERROR(status)) {
@@ -1090,7 +1089,7 @@ static void open4_ex(OPEN4args *arg,
 			state_lock_held = false;
 
 			/* Release the extra LRU reference on file_obj. */
-			file_obj->obj_ops.put_ref(file_obj);
+			file_obj->obj_ops->put_ref(file_obj);
 			goto out;
 		}
 	}
@@ -1159,7 +1158,7 @@ static void open4_ex(OPEN4args *arg,
 
 	if (looked_up_file_obj) {
 		/* We got file_obj via lookup, we need to unref it. */
-		file_obj->obj_ops.put_ref(file_obj);
+		file_obj->obj_ops->put_ref(file_obj);
 	}
 }
 
@@ -1325,7 +1324,7 @@ int nfs4_op_open(struct nfs_argop4 *op, compound_data_t *data,
 	 * current FH.
 	 */
 	obj_change = data->current_obj;
-	obj_change->obj_ops.get_ref(obj_change);
+	obj_change->obj_ops->get_ref(obj_change);
 
 	/* Update the change info for entry_change. */
 	res_OPEN4->OPEN4res_u.resok4.cinfo.before =
@@ -1430,7 +1429,7 @@ int nfs4_op_open(struct nfs_argop4 *op, compound_data_t *data,
 	}
 
 	if (obj_change)
-		obj_change->obj_ops.put_ref(obj_change);
+		obj_change->obj_ops->put_ref(obj_change);
 
 	if (owner != NULL) {
 		/* Need to release the open owner for this call */
