@@ -179,7 +179,7 @@ void cleanup_pseudofs_node(char *pseudopath,
 	cleanup_pseudofs_node(pseudopath, parent_obj);
 
 out:
-	parent_obj->obj_ops.put_ref(parent_obj);
+	parent_obj->obj_ops->put_ref(parent_obj);
 }
 
 bool make_pseudofs_node(char *name, struct pseudofs_state *state)
@@ -188,6 +188,7 @@ bool make_pseudofs_node(char *name, struct pseudofs_state *state)
 	fsal_status_t fsal_status;
 	bool retried = false;
 	struct attrlist sattr;
+	char const *fsal_name;
 
 retry:
 
@@ -204,7 +205,7 @@ retry:
 				state->export->pseudopath,
 				name);
 			/* Release the reference on the new node */
-			new_node->obj_ops.put_ref(new_node);
+			new_node->obj_ops->put_ref(new_node);
 			return false;
 		}
 
@@ -213,7 +214,7 @@ retry:
 			 state->obj, new_node, name,
 			 new_node->fsal->name);
 
-		state->obj->obj_ops.put_ref(state->obj);
+		state->obj->obj_ops->put_ref(state->obj);
 		/* Make new node the current node */
 		state->obj = new_node;
 		return true;
@@ -231,9 +232,11 @@ retry:
 		return false;
 	}
 
-	if (strncmp(op_ctx->ctx_export->fsal_export->exp_ops.get_name(
-				op_ctx->ctx_export->fsal_export),
-			"PSEUDO", 6) != 0) {
+	fsal_name = op_ctx->ctx_export->fsal_export->exp_ops.get_name(
+				op_ctx->ctx_export->fsal_export);
+	/* fsal_name should be "PSEUDO" or "PSEUDO/<stacked-fsal-name>" */
+	if (strncmp(fsal_name, "PSEUDO", 6) != 0 ||
+	    (fsal_name[6] != '/' && fsal_name[6] != '\0')) {
 		/* Only allowed to create directories on FSAL_PSEUDO */
 		LogCrit(COMPONENT_EXPORT,
 			"BUILDING PSEUDOFS: Export_Id %d Path %s Pseudo Path %s LOOKUP %s failed with %s (can't create directory on non-PSEUDO FSAL)",
@@ -284,7 +287,7 @@ retry:
 		 new_node, new_node->state_hdl);
 
 	/* Release reference to the old node */
-	state->obj->obj_ops.put_ref(state->obj);
+	state->obj->obj_ops->put_ref(state->obj);
 
 	/* Make new node the current node */
 	state->obj = new_node;
@@ -401,7 +404,7 @@ bool pseudo_mount_export(struct gsh_export *export)
 			/* Release reference on mount point inode
 			 * and the mounted on export
 			 */
-			state.obj->obj_ops.put_ref(state.obj);
+			state.obj->obj_ops->put_ref(state.obj);
 			put_gsh_export(op_ctx->ctx_export);
 			return false;
 		}
@@ -583,6 +586,6 @@ void pseudo_unmount_export(struct gsh_export *export)
 
 	if (junction_inode != NULL) {
 		/* Release the LRU reference */
-		junction_inode->obj_ops.put_ref(junction_inode);
+		junction_inode->obj_ops->put_ref(junction_inode);
 	}
 }
