@@ -28,6 +28,7 @@
 #include "log.h"
 #include "fsal.h"
 #include "nfs_proto_functions.h"
+#include "nfs_file_handle.h"
 #include "sal_functions.h"
 #include "nlm_util.h"
 #include "nlm_async.h"
@@ -52,7 +53,6 @@ int nlm4_Unshare(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	state_owner_t *nlm_owner;
 	state_t *nlm_state;
 	int rc;
-	bool grace = nfs_in_grace();
 
 	/* NLM doesn't have a BADHANDLE error, nor can rpc_execute deal with
 	 * responding to an NLM_*_MSG call, so we check here if the export is
@@ -68,27 +68,22 @@ int nlm4_Unshare(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 
 	netobj_to_string(&arg->cookie, buffer, 1024);
 
-	LogDebug(COMPONENT_NLM,
-		 "REQUEST PROCESSING: Calling NLM4_UNSHARE cookie=%s reclaim=%s",
-		 buffer,
-		 arg->reclaim ? "yes" : "no");
+	if (isDebug(COMPONENT_NLM)) {
+		char str[LEN_FH_STR];
+		char oh[MAXNETOBJ_SZ * 2] = "\0";
+
+		sprint_fhandle3(str, (struct nfs_fh3 *)&arg->share.fh);
+		netobj_to_string(&arg->share.oh, oh, 1024);
+
+		LogDebug(COMPONENT_NLM,
+			 "REQUEST PROCESSING: Calling NLM4_UNSHARE handle: %s, cookie=%s, reclaim=%s, owner=%s, access=%d, deny=%d",
+			 str, buffer, arg->reclaim ? "yes" : "no", oh,
+			 arg->share.access,
+			 arg->share.mode);
+	}
 
 	copy_netobj(&res->res_nlm4share.cookie, &arg->cookie);
 
-	/* Allow only reclaim share request during recovery and visa versa.
-	 * Note: NLM_SHARE is indicated to be non-monitored, however, it does
-	 * have a reclaim flag, so we will honor the reclaim flag if used.
-	 * This is a little more bizare for UNSHARE, but, we'll still honor
-	 * the reclaim flag.
-	 */
-	if ((grace && !arg->reclaim) || (!grace && arg->reclaim)) {
-		res->res_nlm4share.stat = NLM4_DENIED_GRACE_PERIOD;
-		LogDebug(COMPONENT_NLM,
-			 "REQUEST RESULT: NLM4_UNSHARE %s",
-			 lock_result_str(res->res_nlm4share.stat));
-		return NFS_REQ_OK;
-	}
-
 	rc = nlm_process_share_parms(req,
 				     &arg->share,
 				     op_ctx->fsal_export,
@@ -127,7 +122,7 @@ int nlm4_Unshare(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	dec_nsm_client_ref(nsm_client);
 	dec_nlm_client_ref(nlm_client);
 	dec_state_owner_ref(nlm_owner);
-	obj->obj_ops.put_ref(obj);
+	obj->obj_ops->put_ref(obj);
 	dec_nlm_state_ref(nlm_state);
 
 	LogDebug(COMPONENT_NLM, "REQUEST RESULT: NLM4_UNSHARE %s",
