@@ -16,7 +16,6 @@
 
 char v4_recov_dir[PATH_MAX];
 char v4_old_dir[PATH_MAX];
-char recov_root[PATH_MAX];
 
 /**
  * @brief convert clientid opaque bytes as a hex string for mkdir purpose.
@@ -115,20 +114,18 @@ static void fs_create_clid_name(nfs_client_id_t *clientid)
 		 clientid->cid_recov_tag);
 }
 
-void fs_create_recov_dir(void)
+int fs_create_recov_dir(void)
 {
 	int err;
 
-	snprintf(recov_root, PATH_MAX, "%s", NFS_V4_RECOV_ROOT);
-
-	err = mkdir(recov_root, 0755);
+	err = mkdir(NFS_V4_RECOV_ROOT, 0755);
 	if (err == -1 && errno != EEXIST) {
 		LogEvent(COMPONENT_CLIENTID,
 			 "Failed to create v4 recovery dir (%s), errno=%d",
-			 recov_root, errno);
+			 NFS_V4_RECOV_ROOT, errno);
 	}
 
-	snprintf(v4_recov_dir, sizeof(v4_recov_dir), "%s/%s", recov_root,
+	snprintf(v4_recov_dir, sizeof(v4_recov_dir), "%s/%s", NFS_V4_RECOV_ROOT,
 		 NFS_V4_RECOV_DIR);
 	err = mkdir(v4_recov_dir, 0755);
 	if (err == -1 && errno != EEXIST) {
@@ -137,7 +134,7 @@ void fs_create_recov_dir(void)
 			 v4_recov_dir, errno);
 	}
 
-	snprintf(v4_old_dir, sizeof(v4_old_dir), "%s/%s", recov_root,
+	snprintf(v4_old_dir, sizeof(v4_old_dir), "%s/%s", NFS_V4_RECOV_ROOT,
 		 NFS_V4_OLD_DIR);
 	err = mkdir(v4_old_dir, 0755);
 	if (err == -1 && errno != EEXIST) {
@@ -147,7 +144,7 @@ void fs_create_recov_dir(void)
 	}
 	if (nfs_param.core_param.clustered) {
 		snprintf(v4_recov_dir, sizeof(v4_recov_dir), "%s/%s/node%d",
-			 recov_root, NFS_V4_RECOV_DIR, g_nodeid);
+			 NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR, g_nodeid);
 
 		err = mkdir(v4_recov_dir, 0755);
 		if (err == -1 && errno != EEXIST) {
@@ -157,7 +154,7 @@ void fs_create_recov_dir(void)
 		}
 
 		snprintf(v4_old_dir, sizeof(v4_old_dir), "%s/%s/node%d",
-			 recov_root, NFS_V4_OLD_DIR, g_nodeid);
+			 NFS_V4_RECOV_ROOT, NFS_V4_OLD_DIR, g_nodeid);
 
 		err = mkdir(v4_old_dir, 0755);
 		if (err == -1 && errno != EEXIST) {
@@ -166,6 +163,7 @@ void fs_create_recov_dir(void)
 				 v4_old_dir, errno);
 		}
 	}
+	return 0;
 }
 
 void fs_add_clid(nfs_client_id_t *clientid)
@@ -645,12 +643,12 @@ void fs_read_recov_clids_takeover(nfs_grace_start_t *gsp,
 		break;
 	case EVENT_TAKE_IP:
 		snprintf(path, sizeof(path), "%s/%s/%s",
-			 recov_root, gsp->ipaddr,
+			 NFS_V4_RECOV_ROOT, gsp->ipaddr,
 			 NFS_V4_RECOV_DIR);
 		break;
 	case EVENT_TAKE_NODEID:
 		snprintf(path, sizeof(path), "%s/%s/node%d",
-			 recov_root, NFS_V4_RECOV_DIR,
+			 NFS_V4_RECOV_ROOT, NFS_V4_RECOV_DIR,
 			 gsp->nodeid);
 		break;
 	default:
@@ -779,7 +777,7 @@ void fs_add_revoke_fh(nfs_client_id_t *delr_clid, nfs_fh4 *delr_handle)
 
 struct nfs4_recovery_backend fs_backend = {
 	.recovery_init = fs_create_recov_dir,
-	.recovery_cleanup = fs_clean_old_recov_dir,
+	.end_grace = fs_clean_old_recov_dir,
 	.recovery_read_clids = fs_read_recov_clids_takeover,
 	.add_clid = fs_add_clid,
 	.rm_clid = fs_rm_clid,
