@@ -101,7 +101,7 @@ int rados_urls_set_param_from_conf(void *tree_node,
  */
 
 #define RADOS_URL_REGEX \
-	"([-a-zA-Z0-9_&=]+)/?([-a-zA-Z0-9_&=/]+)?"
+	"([-a-zA-Z0-9_&=.]+)/?([-a-zA-Z0-9_&=/.]+)?"
 
 /** @brief url regex initializer
  */
@@ -129,18 +129,19 @@ static void cu_rados_url_init(void)
 	void *node;
 
 	node = config_GetBlockNode("RADOS_URLS");
-	if (!node) {
+	if (node) {
+		ret = rados_urls_set_param_from_conf(node, &err_type);
+		if (ret < 0) {
+			LogEvent(COMPONENT_CONFIG,
+				"%s: Failed to parse RADOS_URLS %d",
+				__func__, ret);
+		}
+	} else {
 		LogWarn(COMPONENT_CONFIG,
-			"%s: Failed to lookup RADOS_URLS config block",
+			"%s: RADOS_URLS config block not found",
 			__func__);
-		return;
 	}
 
-	ret = rados_urls_set_param_from_conf(node, &err_type);
-	if (ret < 0) {
-		LogEvent(COMPONENT_CONFIG, "%s: Failed to parse RADOS_URLS %d",
-			__func__, ret);
-	}
 
 	ret = rados_create(&cluster, rados_url_param.userid);
 	if (ret < 0) {
@@ -196,7 +197,7 @@ static inline char *match_dup(regmatch_t *m, char *in)
 static int cu_rados_url_fetch(const char *url, FILE **f, char **fbuf)
 {
 	rados_ioctx_t io_ctx;
-	char *x0, *x1, *x2;
+	char *x0 = NULL, *x1 = NULL, *x2 = NULL;
 
 	char *pool_name;
 	char *object_name;
