@@ -121,9 +121,6 @@ struct gsh_export {
 	uint32_t options;
 	/** CFG: Export non-permission options set - atomic changeable option */
 	uint32_t options_set;
-	/** CFG: Expiration time interval in seconds for attributes.  Settable
-	    with Attr_Expiration_Time. - atomic changeable option */
-	int32_t expire_time_attr;
 	/** CFG: Export_Id for this export - static option */
 	uint16_t export_id;
 
@@ -208,6 +205,7 @@ void _put_gsh_export(struct gsh_export *a_export,
 	_put_gsh_export(a_export, \
 	(char *) __FILE__, __LINE__, (char *) __func__)
 
+void export_cleanup(struct gsh_export *a_export);
 void export_revert(struct gsh_export *a_export);
 void export_add_to_mount_work(struct gsh_export *a_export);
 void export_add_to_unexport_work_locked(struct gsh_export *a_export);
@@ -220,5 +218,6 @@ extern struct config_block update_export_param;
 
 void remove_all_exports(void);
 
+extern struct timespec nfs_stats_time;
 #endif				/* !EXPORT_MGR_H */
 /** @} */
