@@ -111,12 +111,14 @@ static const uint32_t FSAL_UP_INVALIDATE_CONTENT = 0x04;
 static const uint32_t FSAL_UP_INVALIDATE_DIR_POPULATED = 0x08;
 static const uint32_t FSAL_UP_INVALIDATE_DIR_CHUNKS = 0x10;
 static const uint32_t FSAL_UP_INVALIDATE_CLOSE = 0x100;
+static const uint32_t FSAL_UP_INVALIDATE_FS_LOCATIONS = 0x200;
 #define FSAL_UP_INVALIDATE_CACHE ( \
 	FSAL_UP_INVALIDATE_ATTRS | \
 	FSAL_UP_INVALIDATE_ACL | \
 	FSAL_UP_INVALIDATE_CONTENT | \
 	FSAL_UP_INVALIDATE_DIR_POPULATED | \
-	FSAL_UP_INVALIDATE_DIR_CHUNKS)
+	FSAL_UP_INVALIDATE_DIR_CHUNKS | \
+	FSAL_UP_INVALIDATE_FS_LOCATIONS)
 
 /**
  * @brief Possible upcall functions
