@@ -154,6 +154,12 @@ typedef enum protos {
  */
 #define NFS_DEFAULT_RECV_BUFFER_SIZE 1048576
 
+/**
+ * @brief Turn off all protocols
+ */
+
+#define CORE_OPTION_NONE 0x00000000	/*< No operations are supported */
+
 /**
  * @brief Support NFSv3
  */
@@ -198,8 +204,8 @@ typedef struct nfs_core_param {
 	/** An array of port numbers, one for each protocol.  Set by
 	    the NFS_Port, MNT_Port, NLM_Port, and Rquota_Port options. */
 	uint16_t port[P_COUNT];
-	/** The address to which to bind for our listening port.
-	    IPv4 only, for now.  Set by the Bind_Addr option. */
+	/** The IPv4 or IPv6 address to which to bind for our
+	    listening port.  Set by the Bind_Addr option. */
 	struct sockaddr_storage bind_addr;
 	/** An array of RPC program numbers.  The correct values, by
 	    default, they may be set to incorrect values with the
