@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python2
 #
 # ganesha_mgr_utils.py - commandline tool utils for managing nfs-ganesha.
 #
@@ -268,6 +268,29 @@ class AdminInterface():
         msg = reply[1]
         return status, msg
 
+    def purge_idmap(self):
+        method = self.dbusobj.get_dbus_method("purge_idmapper_cache",
+                                              self.dbus_interface)
+        try:
+           reply = method()
+        except dbus.exceptions.DBusException as e:
+           return False, e
+
+        status = reply[0]
+        msg = reply[1]
+        return status, msg
+
+    def purge_gids(self):
+        method = self.dbusobj.get_dbus_method("purge_gids",
+                                              self.dbus_interface)
+        try:
+           reply = method()
+        except dbus.exceptions.DBusException as e:
+           return False, e
+
+        status = reply[0]
+        msg = reply[1]
+        return status, msg
 
 LOGGER_PROPS = 'org.ganesha.nfsd.log.component'
 
