@@ -1,12 +1,13 @@
-#!/usr/bin/python
+#!/usr/bin/python2
 
 # You must initialize the gobject/dbus support for threading
 # before doing anything.
+from __future__ import print_function
 import gobject
 import sys
 
 ipaddr=sys.argv[1]
-print 'event:ip_addr=', ipaddr
+print('event:ip_addr=', ipaddr)
 
 gobject.threads_init()
 
@@ -29,9 +30,9 @@ except dbus.exceptions.DBusException as e:
 ganesha_grace = admin.get_dbus_method('grace',
                                'org.ganesha.nfsd.admin')
 
-print "Start grace period."
+print("Start grace period.")
 
 try:
-	print ganesha_grace(ipaddr)
+	print(ganesha_grace(ipaddr))
 except dbus.exceptions.DBusException as e:
 	sys.exit("Error: Failed to start grace period")
