@@ -167,7 +167,7 @@ int nfs_ip_name_add(sockaddr_t *ipaddr, char *hostname, size_t size)
 	 */
 	memcpy(pipaddr, ipaddr, sizeof(sockaddr_t));
 
-	buffkey.addr = (caddr_t) pipaddr;
+	buffkey.addr = pipaddr;
 	buffkey.len = sizeof(sockaddr_t);
 
 	gettimeofday(&tv0, NULL);
@@ -204,7 +204,7 @@ int nfs_ip_name_add(sockaddr_t *ipaddr, char *hostname, size_t size)
 	 */
 	nfs_ip_name->timestamp = time(NULL);
 
-	buffdata.addr = (caddr_t) nfs_ip_name;
+	buffdata.addr = nfs_ip_name;
 	buffdata.len = sizeof(nfs_ip_name_t);
 
 	if (HashTable_Set(ht_ip_name, &buffkey, &buffdata) != HASHTABLE_SUCCESS)
@@ -237,7 +237,7 @@ int nfs_ip_name_get(sockaddr_t *ipaddr, char *hostname, size_t size)
 
 	sprint_sockip(ipaddr, ipstring, sizeof(ipstring));
 
-	buffkey.addr = (caddr_t) ipaddr;
+	buffkey.addr = ipaddr;
 	buffkey.len = sizeof(sockaddr_t);
 
 	if (HashTable_Get(ht_ip_name,
@@ -276,7 +276,7 @@ int nfs_ip_name_remove(sockaddr_t *ipaddr)
 
 	sprint_sockip(ipaddr, ipstring, sizeof(ipstring));
 
-	buffkey.addr = (caddr_t) ipaddr;
+	buffkey.addr = ipaddr;
 	buffkey.len = sizeof(sockaddr_t);
 
 	if (HashTable_Del(ht_ip_name, &buffkey, NULL, &old_value) ==
